import Vue from 'vue';

const LOCALE = 'locale';
const DEFAULT_LOCALE = 'defaultLocale';
const { defineReactive } = Vue.util;
let translations;
const getValue = (input, key) => {
    key = key.replace(/\[(\d+)]/g, '.$1');
    let value = input;
    // eslint-disable-next-line array-callback-return
    key.split('.').some(k => {
        if (!value || typeof value !== 'object') {
            return true;
        }
        value = value[k];
    });
    if (typeof value === 'object') {
        // istanbul ignore next
        if (process.env.NODE_ENV === 'development' && value !== null) {
            Vue.util.warn('you are trying to get non-literal value');
        }
        return value && String(value);
    }
    return value;
};
const createTranslator = (translatorOptions) => {
    if (typeof translatorOptions === 'string') {
        translatorOptions = { locale: translatorOptions };
    }
    const { locale: instanceLocale, translations: instanceTranslations, defaultLocale: instanceDefaultLocale, } = translatorOptions;
    if (instanceTranslations) {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (!translations) {
            translations = instanceTranslations;
        }
        // istanbul ignore next
        else if (process.env.NODE_ENV === 'development' &&
            translations !== instanceTranslations) {
            Vue.util.warn('translations should only be injected once!');
        }
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    }
    else if (process.env.NODE_ENV === 'development' && !translations) {
        Vue.util.warn('translations has not be injected, translator will not work!');
    }
    const instance = (key, params, ignoreNonExist) => {
        const { locale } = instance;
        const translation = translations[locale];
        let value = getValue(translation, key);
        if (value === undefined) {
            const { defaultLocale } = instance;
            if (defaultLocale && defaultLocale !== locale) {
                const defaultTranslation = translations[defaultLocale];
                value = getValue(defaultTranslation, key);
            }
            if (process.env.NODE_ENV === 'development' &&
                value === undefined &&
                !ignoreNonExist) {
                Vue.util.warn(`your are trying to get nonexistent key \`${key}\` without default locale fallback`);
            }
        }
        value = value === null || value === void 0 ? void 0 : value.replace(/{([^{}]+)}/g, (_matched, $0) => getValue(params, $0.trim()));
        return value == null ? key : value;
    };
    defineReactive(instance, LOCALE, instanceLocale);
    defineReactive(instance, DEFAULT_LOCALE, instanceDefaultLocale);
    return instance;
};

const mergedCache = [];
// eslint-disable-next-line sonar/function-name
const VueTranslator = ($Vue, options) => {
    if (typeof options === 'string') {
        options = {
            locale: options,
        };
    }
    const { defaultLocale, locale, merge, translations = {} } = options;
    const defaultTranslator = createTranslator({
        locale,
        translations,
        defaultLocale,
    });
    $Vue.translator = defaultTranslator;
    $Vue.mixin({
        beforeCreate() {
            const { translator } = this.$options;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            const { cid } = Object.getPrototypeOf(this).constructor;
            if (!translator || mergedCache.indexOf(cid) + 1) {
                return;
            }
            if (!merge) {
                // istanbul ignore next
                if (process.env.NODE_ENV === 'development') {
                    $Vue.util.warn('VueTranslator will not help you to merge translations, please pass your own merge strategy, `lodash.merge` for example');
                }
                return;
            }
            merge(translations, translator);
            mergedCache.push(cid);
        },
    });
    Object.defineProperty($Vue.prototype, '$t', process.env.VUE_ENV === 'server'
        ? {
            get() {
                const { translator } = this.$ssrContext;
                if (process.env.NODE_ENV === 'development' && !translator) {
                    $Vue.util.warn('there is no translator instance on ssrContext, you should register it manually');
                }
                return translator || defaultTranslator;
            },
            configurable: process.env.NODE_ENV === 'development',
        }
        : {
            value: defaultTranslator,
            writable: process.env.NODE_ENV === 'development',
        });
    let { filter } = options;
    if (!filter) {
        return;
    }
    if (filter === true) {
        filter = 'translate';
    }
    if (process.env.VUE_ENV !== 'server') {
        $Vue.filter(filter, defaultTranslator);
        return;
    }
    const { _f } = $Vue.prototype;
    // a hacky way to support filter on server, so `filter` is not enabled by default
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    $Vue.prototype._f = function (id) {
        var _a;
        if (process.env.NODE_ENV === 'development' &&
            ((_a = this.$options.filters) === null || _a === void 0 ? void 0 : _a[filter])) {
            $Vue.util.warn(`duplicate filter \`${filter}\` found, please rename to a unique filter name`);
        }
        return id === filter ? this.$t : _f.call(this, id);
    };
};
const install = VueTranslator;

export default VueTranslator;
export { createTranslator, install };
